@@ -544,6 +544,11 @@ class Aliases(ModuleAnalysis):
 
     # aliasing created by statements
 
+    def init_function_alias(self, node):
+        "each argument is bound to a different identifier"
+        self.aliases.update((arg.id, {arg})
+                            for arg in node.args.args)
+
     def visit_FunctionDef(self, node):
         '''
         Initialise aliasing default value before visiting.
@@ -558,8 +563,7 @@ class Aliases(ModuleAnalysis):
         self.aliases.update((k, {v})
                             for k, v in self.global_declarations.items())
 
-        self.aliases.update((arg.id, {arg})
-                            for arg in node.args.args)
+        self.init_function_alias(node)
 
         self.generic_visit(node)
         if Aliases.RetId in self.aliases:
@@ -612,6 +616,27 @@ class Aliases(ModuleAnalysis):
 
             node.return_alias = merge_return_aliases
 
+    def visit_Assert(self, node):
+        self.generic_visit(node)
+
+        if not isinstance(node.test, ast.Compare):
+            return
+        if len(node.test.ops) != 1:
+            return
+        op = node.test.ops[0]
+        comparator = node.test.comparators[0]
+
+        if not isinstance(node.test.left, ast.Name):
+            return
+        if not isinstance(comparator, ast.Name):
+            return
+
+        if isinstance(op, ast.IsNot):
+            left_aliases = self.aliases[node.test.left.id]
+            right_aliases = self.aliases[comparator.id]
+            self.aliases[node.test.left.id] = self.aliases[node.test.left.id].difference(right_aliases)
+            right_aliases.difference_update(left_aliases)
+
     def visit_Assign(self, node):
         r'''
         Assignment creates aliasing between lhs and rhs
@@ -626,8 +651,9 @@ class Aliases(ModuleAnalysis):
         Everyone points to the formal parameter 'a' \o/
         '''
         md.visit(self, node)
-        value_aliases = self.visit(node.value)
-        for t in node.targets:
+        value_aliases = self.visit(node.value) if node.value else {}
+        targets = node.targets if isinstance(node, ast.Assign) else (node.target,)
+        for t in targets:
             if isinstance(t, ast.Name):
                 self.aliases[t.id] = set(value_aliases) or {t}
                 for alias in list(value_aliases):
@@ -638,6 +664,8 @@ class Aliases(ModuleAnalysis):
             else:
                 self.visit(t)
 
+    visit_AnnAssign = visit_Assign
+
     def visit_For(self, node):
         '''
         For loop creates aliasing between the target
@@ -786,3 +814,14 @@ class StrictAliases(Aliases):
 
     def get_unbound_value_set(self):
         return set()
+
+
+class InterproceduralAliases(Aliases):
+    """
+    Gather aliases while assuming two different parameters can point to the same
+    value
+    """
+
+    def init_function_alias(self, node):
+        self.aliases.update((arg.id, set(node.args.args))
+                            for arg in node.args.args)
